/* delay.h {\documentstyle{jsrc2latex}\rhead{\sl delay.h\quad\thepage}
   \font\tenrm=cmtt10}
{\ \begin{verbatim}
$RCSfile: delay.h,v $  $Revision: 1.2 $
$Date: 1996/08/15 10:02:53 $  by KyotoHydro
\end{verbatim}}
{\ {\setlength{\parskip}{4pt}% latex-mode
}% end of latex-mode }

Time delay element model which receives Data<T> and outputs it at a
delayed time by "timeLag."

The output at the initial time must be given as one of the initial state
variables.

*/

#if !defined(delay_h)
#define delay_h

#include <stdio.h>
#include <time.h>
#include <assert.h>

#include "element.h"                      // definition of class Element
#include "p_tmp.h"         // definition of template class Receive, Send

/* ---------------------------------------------------------------------
CLASS
   Delay&<T&>

   Time delay element model

KEYWORDS

   time delay

DESCRIPTION

   Time delay element model which receives Data<T> and outputs it at a
   delayed time by "timeLag."

   [receive ports] One receive port of type ReceiveNointp<T>. Its object
   name is "rp" and object number is 0.

   [send port] One send port of type Send<T>. Its object name is "sp"
   and its object number is 0.

----------------------------------------------------------------------*/

template <class T>
class Delay : public Element {
	/*
    ## class Delay: Time delay element model
	*/

//  1. Declare receive ports

protected:
  //////////////////////////////////////////////////////////////////////
  // Receive port.
  ReceiveNointp<T> rp;

//  2. Define member function Register_receive_ports(void) which
//     registers receive ports

protected:
  //////////////////////////////////////////////////////////////////////
  void Register_receive_ports(void) {
	    rp.Init("rp", 0);
	    Register((void*) &rp);
  }

//  3. Declare send ports

protected:
  //////////////////////////////////////////////////////////////////////
  // Send port
  Send<T> sp;                                   // output

//  4. Define member function Register_send_ports(void) which registers
//     send ports

  void Register_send_ports(void) {
		sp.Init("sp", 0);
		Register((void*) &sp);
  }

//  5. Declare parameters as data members

protected:
  //////////////////////////////////////////////////////////////////////
  // timeLag parameter. timeLag must not be negative.
  long timeLag;

//  6. Declare state variables as data members

protected:
  //////////////////////////////////////////////////////////////////////
  // the newest data value which was sent to "sp".
  T newestValue;

//  7. Declare working variables if needed

    // No need for working variables for the class

//  8. Define member function Set_parameter(FILE* fp) which reads in
//     parameter values from a file

public:
  //////////////////////////////////////////////////////////////////////
  // Reads in parameter value <b>timeLag</b> from a file.
  //
  // scanf(fp, "%ld", &timeLag);
  void Set_parameter(FILE* fp);

//  9. Define member function Set_parameter(...) which sets parameter
//     values from its arguments

public:
  //////////////////////////////////////////////////////////////////////
  // Sets the parameter value <b>timeLag</b> to <b>timeLag_z<b>
  void Set_parameter(long timeLag_z) {
		timeLag = timeLag_z;
		assert(timeLag >= 0);
  }

// 10. Define member function Set_initial_state(FILE* fp) which reads in
//     initial values of the state variables from a file

  //////////////////////////////////////////////////////////////////////
  // Reads in initial values of the state variables from a file.
  // current_time and data value at current_time are the state variables.
  // In the file fp, current_time and the data value must be given in
  // one line.
  void Set_initial_state(FILE* fp);

// 11. Define member function Set_initial_state(...) which sets initial
//     values of the state variables from its arguments

  //////////////////////////////////////////////////////////////////////
  // Sets initial values of the state variables from its arguments.
  //
  // [in] cp_z - current_time
  // [in] newestValue_z - newest value
  //
  void Set_initial_state(time_t cp_z, T newestValue_z) {
		if (cp_z > 0) current_time = cp_z;
		newestValue = newestValue_z;
  }

// 12. Define member function Save_terminal_state(FILE* fp) which writes
//     terminal states into a file

  //////////////////////////////////////////////////////////////////////
  // Writes terminal states, that is, current time and newestValue to a
  // file.
  //
  // [in] fp - file pointer to the file
  void Save_terminal_state(FILE *fp) {
		if (fp != NULL) {
		    data2ofile(current_time, fp);
			fprintf(fp, " ");
			data2ofile(newestValue, fp);
		}
	}

// 13. Define member function Save_terminal_state(...) which retrieve
//     terminal states into its arguments

  //////////////////////////////////////////////////////////////////////
  // Get terminal states, that is, current time and newestValue to
  // arguments.
  //
  // [out] ct     - current_time
  // [out] value  - newestValue
  void Save_terminal_state(time_t& ct, T& value) {
		ct = current_time;
		value = newestValue;
	}

// 14. Declare the substition operator

	// Declaration of substitution operator for prohibiting
	// substitution.  Implementation of this operator should not be
	// given.

private:
  //////////////////////////////////////////////////////////////////////
  // In oder to prohibit the execution of operator =, only the
  // declaration is given here.
  Delay<T>& operator=(const Delay<T>& rhs);

// 15. Constructor and destructor

public:

  //////////////////////////////////////////////////////////////////////
  // Sets the object name to <b>o_name</b> and the object number to
  // <b>o_num</b>
  Boolean Init(char* o_name, int o_num) {
		return Element::Init(o_name, o_num);
	}

  //////////////////////////////////////////////////////////////////////
  // Constructor.
  // Sets the object name to <b>o_name</b> and the object number to
  // <b>o_num</b>
  Delay(char* o_name, int o_num) : Element() {
		class_name = "Delay";
		Init(o_name, o_num);
	}

  //////////////////////////////////////////////////////////////////////
  // Constructor.
  Delay(void) : Element() {
		class_name = "Delay";
  }

  //////////////////////////////////////////////////////////////////////
  // Destructor
  ~Delay(void) {};

// 16. Define member function Initial_output

protected:

  //////////////////////////////////////////////////////////////////////
  // Initial output
  Boolean Initial_output(void);

// 17. Define member function Calculate_time_step

protected:

  //////////////////////////////////////////////////////////////////////
  // Calculate time step.
  time_t Calculate_time_step(void);

// 18. Define member function Can_you_calculate

protected:

  //////////////////////////////////////////////////////////////////////
  Boolean Can_you_calculate(void) { return YES; }

	// This function is not called from Delay::Work. However, in the
	// base class this member function is declared as pure virtual.
	// Therefore, we define  as above as an expedient.

// 19. Define member function Calculate

protected:

  //////////////////////////////////////////////////////////////////////
  Boolean Calculate(void);

// 20. Define member function Work

public:

  //////////////////////////////////////////////////////////////////////
  Boolean Work(void);

// 21. Define member function Necessary_time_from

public:

  //////////////////////////////////////////////////////////////////////
  time_t Necessary_time_from(Receive_port*);

	// In default, this member function returns current_time. However,
	// the Delay class needs different definition than the
	// default.  The data before the time which this function returns
	// are regarded as not needed for further calculation.

};

#endif

/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.2 $
Change in Revision 1.2
Bug fix

# Local Variables: ***
# tab-width: 4 ***
# End: ***
\end{verbatim}\end{comment}
} */
